/*
 * UHUPolyCanvas.java
 *
 * Created on 26 de noviembre de 2007, 11:02
 */

package interfaz;

import java.awt.Color;
import java.awt.Image;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.*;
import java.beans.*;
import java.io.Serializable;
import javax.swing.ImageIcon;
import javax.swing.JPanel;

/**
 * @author Pandre
 */
public class UHUPolyCanvas extends JPanel implements Serializable {
    
    private Image imgCasillas[] = new Image[40];
    private Image imgJugadores[] = new Image[6];
    private Image imgCasas[] = new Image[4];
    private Image imgHoteles[] = new Image[4];
    private Image imgPestanasPropiedad[] = new Image[7];

    private Image dbDibujo = null;  // imagen para el doble bfer
    private Graphics dbDibujo_g;    // contexto grfico para el doble bfer

    public void inicializar() {
        
        for (int i = 0; i < 40; i++)
            imgCasillas[i] = new ImageIcon("res/casilla" + (i+1) + ".png").getImage();
        
        for (int i = 0; i < 6; i++)
            imgJugadores[i] = new ImageIcon("res/jugador" + (i+1) + ".png").getImage();
        
        for (int i = 0; i < 4; i++)
            imgCasas[i] = new ImageIcon("res/casa" + (i+1) + ".png").getImage();
        
        for (int i = 0; i < 4; i++)
            imgHoteles[i] = new ImageIcon("res/hotel" + (i+1) + ".png").getImage();
        
        for (int i = 0; i < 7; i++)
            imgPestanasPropiedad[i] = new ImageIcon("res/pestanaPropiedad" + (i) + ".png").getImage();
        
        if (dbDibujo == null)
            dbDibujo = createImage(getWidth(), getHeight());

        
        dbDibujo_g = dbDibujo.getGraphics();
        dbDibujo_g.setColor(Color.white);
        dbDibujo_g.fillRect(0, 0, getWidth(), getHeight());

        for (int i = 1; i <= 40; i++)
            pintarCasilla(i);
        
        pintaLogo(false);
    
    }

    public void update(Graphics g) {
        // redefinimos para que slo pinte en paintComponent y no borre todo.
        paintComponent(g);
    }

    public void paintComponent(Graphics g) {
        if (dbDibujo != null)
            g.drawImage(dbDibujo, 0, 0, this);  // copia la imagen del doble bfer en el componente
    }
    
    private Posicion obtenerPos(int numCasilla) {
        int posiciones[][] = {{630,630},{570,630},{510,630},{450,630},{390,630},
                {330,630},{270,630},{210,630},{150,630},{90,630}, {0,630},
                {0,570},{0,510},{0,450},{0,390},{0,330},{0,270},{0,210},{0,150},
                {0,90},{0,0},{90,0},{150,0},{210,0},{270,0},{330,0},{390,0},
                {450,0},{510,0},{570,0},{630,0},{630,90},{630,150},{630,210},
                {630,270},{630,330},{630,390},{630,450},{630,510},{630,570}};
        
        int orient;     
        if ((numCasilla >=2) && (numCasilla <= 11)) orient = 0;
        else if ((numCasilla >=12) && (numCasilla <= 20)) orient = 1;
        else if ((numCasilla >=21) && (numCasilla <= 30)) orient = 2;
        else orient = 3;
        Posicion res = new Posicion(posiciones[numCasilla-1][0], posiciones[numCasilla-1][1], orient);
        return res;
    }
    
    public void pintaLogo(boolean aclarado) {
    
        Image logo;
        if (aclarado)
            logo = new ImageIcon("res/logotipoablend.png").getImage();
        else
            logo = new ImageIcon("res/logotipo.png").getImage();
        
        dbDibujo_g = dbDibujo.getGraphics();

        dbDibujo_g.drawImage(logo, 134, 238, Color.white, this);

        repaint(); 
    
    }

    public void pintarCasilla(int numCasilla) {

        Posicion pos = obtenerPos(numCasilla);
        dbDibujo_g = dbDibujo.getGraphics();

        dbDibujo_g.drawImage(imgCasillas[numCasilla-1], pos.getX(), pos.getY(), Color.white, this);

        repaint();  
        
    }
    
    
    public void pintarJugador(int jugador, int numCasilla) {
            // [orient][x][y]
        int desp[][][] = {{{8,23},{34,23},{8,45},{34,45},{8,67},{34,67}}, // abajo
                         {{3,10},{25,10},{47,10},  {3,32},{25,32},{47,32}},  // izq
                         {{11,9},{32,9},{11,28},{32,28},{11,47},{32,47}},  // arriba
                         {{23,10},{45,10},{67,10},  {23,32},{45,32},{67,32}}}; // der
        
        Posicion pos = obtenerPos(numCasilla);
        dbDibujo_g = dbDibujo.getGraphics();

        dbDibujo_g.drawImage(imgJugadores[jugador-1], pos.getX()+desp[pos.getOrient()][jugador-1][0],
                             pos.getY()+desp[pos.getOrient()][jugador-1][1], this);

        repaint();  
        
    }
    
    public void pintarCasa(int numCasas, int numCasilla) {
            // [orient][x][y]
        int desp[][][] = {{{2,3},{16,3},{30,3},{44,3}}, // abajo
                         {{73,2},{73,16},{73,30},{73,44}},  // izq
                         {{2,73},{16,73},{30,73},{44,73}},  // arriba
                         {{3,2},{3,16},{3,30},{3,44}}}; // der
        
        Posicion pos = obtenerPos(numCasilla);
        dbDibujo_g = dbDibujo.getGraphics();

        for (int i = 0; i < numCasas; i++)
            dbDibujo_g.drawImage(imgCasas[pos.getOrient()], pos.getX()+desp[pos.getOrient()][i][0],
                             pos.getY()+desp[pos.getOrient()][i][1], this);

        repaint();  
        
    }
    
    public void pintarHotel(int numCasilla) {
            // [orient][x][y]
        int desp[][][] = {{{2,3}}, // abajo
                          {{73,2}},  // izq
                          {{2,73}},  // arriba
                          {{3,2}}}; // der
        
        Posicion pos = obtenerPos(numCasilla);
        dbDibujo_g = dbDibujo.getGraphics();

        dbDibujo_g.drawImage(imgHoteles[pos.getOrient()], pos.getX()+desp[pos.getOrient()][0][0],
                             pos.getY()+desp[pos.getOrient()][0][1], this);

        repaint();  
        
    }
    
    public void pintarPropiedad(int numCasilla, int numJugador) {
            // [orient][x][y]
        int desp[][][] = {{{-15,-48}}, // abajo
                          {{50,-15}},  // izq
                          {{-13,50}},  // arriba
                          {{-48,-13}}}; // der
        
        Posicion pos = obtenerPos(numCasilla);
        dbDibujo_g = dbDibujo.getGraphics();

        dbDibujo_g.drawImage(rotarImagen(imgPestanasPropiedad[numJugador], pos.getOrient()*90-90).getImage(), pos.getX()+desp[pos.getOrient()][0][0],
                             pos.getY()+desp[pos.getOrient()][0][1], this);

        repaint();  
        
    }
    
    /*
    * Funcin tomada y adaptada desde el ejemplo mostrado en:
    *   http://saloon.javaranch.com/cgi-bin/ubb/ultimatebb.cgi?ubb=get_topic&f=2&t=013741
    */
    private ImageIcon rotarImagen(Image img, int grados)
    {
        // Carga la imagen
        BufferedImage imagen = null;
        /*try
        {
            imagen = ImageIO.read(new File("res\\casilla" + numCasilla + ".png"));
        }
        catch(IOException ioe) {}*/
        imagen = toBufferedImage(img);
        
        // Imagen resultante
        ImageIcon icono = new ImageIcon();
        {
            // Creamos una imagen temporal en memoria
            BufferedImage bi = new BufferedImage(90, 90, BufferedImage.TYPE_INT_ARGB);
            Graphics2D g2 = bi.createGraphics();

            // Definimos la transformacin
            double angulo = Math.toRadians(grados);
            double x = (90 - imagen.getWidth())/2;
            double y = (90 - imagen.getHeight())/2;
            AffineTransform at = AffineTransform.getTranslateInstance(x,y);
            at.rotate(angulo, imagen.getWidth()/2, imagen.getHeight()/2);
            
            // Dibujamos la imagen original con la transformacin en la temporal
            g2.drawRenderedImage(imagen, at);
            
            // Liberamos la memoria usada
            g2.dispose();
            
            // Devolvemos la imagen rotada
            icono = new ImageIcon(bi);
        }

        return icono;
    }
    
    /*
    * Funcin tomada y adaptada desde el ejemplo mostrado en:
    *   http://www.exampledepot.com/egs/java.awt.image/Image2Buf.html
    */
    public static BufferedImage toBufferedImage(Image image) {
        if (image instanceof BufferedImage) {
            return (BufferedImage)image;
        }
    
        // This code ensures that all the pixels in the image are loaded
        image = new ImageIcon(image).getImage();
    
        // Determine if the image has transparent pixels; for this method's
        // implementation, see e661 Determining If an Image Has Transparent Pixels
       // boolean hasAlpha = hasAlpha(image);
    
        // Create a buffered image with a format that's compatible with the screen
        BufferedImage bimage = null;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try {
            // Determine the type of transparency of the new buffered image
            int transparency = Transparency.OPAQUE;
          //  if (hasAlpha) {
                transparency = Transparency.BITMASK;
            //}
    
            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bimage = gc.createCompatibleImage(
                image.getWidth(null), image.getHeight(null), transparency);
        } catch (HeadlessException e) {
            // The system does not have a screen
        }
    
        if (bimage == null) {
            // Create a buffered image using the default color model
            int type = BufferedImage.TYPE_INT_RGB;
          //  if (hasAlpha) {
                type = BufferedImage.TYPE_INT_ARGB;
            //}
            bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
        }
    
        // Copy image to buffered image
        Graphics g = bimage.createGraphics();
    
        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
    
        return bimage;
    }

    
    class Posicion {
        private int xx;
        private int yy;
        private int orientacion;
        Posicion( int x, int y , int o){
            this.xx = x;
            this.yy = y;
            this.orientacion = o;
        }
        public int getX() { return xx; }
        public int getY() { return yy; }
        public int getOrient() { return orientacion; }
    }
    
}